#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* Stores all information in a record, so the recordwindow won't fail or crash
 * in case the main window has switched to a different table. Different
 * database is too complicated : in that case we should even consider to 
 * destroy all open recordwindows. (but what if table is changed back ?
 * database will issue errors or warnings anyway !) */
Record *newRecordstruct(TabInfo *ti)
{
   int y;
   int tlen;
   Record *rec;
   GtkTreeIter iter;
   GtkTreePath *path;
   gchar *text;
   Params *p = getParams();
   
   ldebug(2,"newRecordstruct entered\n");
   
   if(ti->inuse != TRUE)
   {
      fprintf(stderr,"newRecordstruct: trouble ahead, ti->inuse != TRUE !!\n");
      return NULL;
   }

   rec = (Record *)g_malloc(sizeof(Record));
   if(rec == NULL)
     return NULL;

   rec->numflds    = ti->numflds;
   rec->dbsock     = ti->dbsock;
   rec->sortkey    = p->sortkey;
   /* sortkey may have changed after creation of a tabinfo ! */

   tlen = strlen(ti->table) + 1;
   rec->table = (char *)g_malloc(tlen);
   strncpy(rec->table,ti->table,tlen);

   /* allocate content buffers for later use : */
   rec->contents = (char **)g_malloc(sizeof(char *) * ti->numflds);
   
   /* Copy complete field array from TabInfo : */
   rec->fields=copyFields(ti->fields,ti->numflds);
   
   /* Copy selected tab contents (if any) to record structure
    * (only for Modify, Copy & Restore):
    */
   if(ti->selectedrow != -1)
   {
      path = gtk_tree_path_new_from_indices (ti->selectedrow,-1);
      gtk_tree_model_get_iter(GTK_TREE_MODEL (ti->list),&iter,path);
   }
   
   for(y=0;y<ti->numflds;y++)
   {
      tlen = ti->fields[y].length + 1;
      rec->contents[y] = (char *)g_malloc(tlen);
      if(ti->selectedrow != -1)
      {
	 gtk_tree_model_get (GTK_TREE_MODEL (ti->list), &iter, y, &text, -1);
	 strncpy(rec->contents[y],text,tlen);
      }
   }
   
   /* do we still need this ? yes : for reading the changed fieldcontents */
   rec->entries = (GtkEntry **)g_malloc(sizeof(GtkEntry *) * ti->numflds);

   rec->window = NULL; /* NULL until topwindow newd */
   return rec;
}

Record *copyRecordstruct(Record *src)
{
   int y;
   int tlen;
   Record *dst;
   
   ldebug(2,"copyRecordstruct entered\n");
   
   if(src == NULL)
     return NULL;
   dst = (Record *)g_malloc(sizeof(Record));
   if(dst == NULL)
     return NULL;

   dst->numflds    = src->numflds;
   dst->dbsock     = src->dbsock;
   dst->sortkey    = src->sortkey;

   tlen = strlen(src->table) + 1;
   dst->table = (char *)g_malloc(tlen);
   strncpy(dst->table,src->table,tlen);

   /* copy fieldnames and new contentbuffer for later use : */

   dst->contents = (char **)g_malloc(sizeof(char *) * src->numflds);

   /* Copy complete field array from src : */
   dst->fields = copyFields(src->fields,src->numflds);

   for(y=0;y<src->numflds;y++)
   {
      tlen = src->fields[y].length + 1;
      dst->contents[y] = (char *)g_malloc(tlen);
      /* don't copy from entries but from contents so we'll get changed 
       * fields in the source recordwindow as well : */
      strncpy(dst->contents[y],gtk_entry_get_text(src->entries[y]),tlen);
   }
   
   /* do we still need this ? yes : for reading the changed fieldcontents */
   dst->entries = (GtkEntry **)g_malloc(sizeof(GtkEntry *) * src->numflds);

   return dst;
}

void destroyRecordstruct(Record *rec)
{
   int y;
   
   ldebug(2,"destroyRecordstruct entered\n");
   
   if(rec == NULL)
     return;

   freeFields(rec->fields,rec->numflds);
   
   for(y=0; y<rec->numflds; y++)
      g_free(rec->contents[y]);
   g_free(rec->contents);

   g_free(rec->table);
   g_free(rec->entries);
   
   g_free(rec);
}
