#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

void releaseFieldInfo(void)
{
   Params *p = getParams();
   int x;
   
   ldebug(2,"releaseFieldInfo called\n");
   if(p->fields == NULL)
     return;
   
   for(x=0;x<p->numflds;x++)
     g_free(p->fields[x].name);
   g_free(p->fields);
   p->fields = NULL;
   
   setGUICounters(0,0);
}

void getFieldInfo(void)
{
   int   x;
   int tlen;
   SQL_RES *res;
   Params *p = getParams();
   
   ldebug(2,"getFieldInfo called\n");
   releaseFieldInfo();
   
   res = sql_list_fields(p->dbsock,p->table, NULL);
   if (res == NULL)
   {
      showSqlError(p->dbsock,"Cannot fetch fieldnames for %s",p->table);
      p->table[0] = 0;
      return;
   }
   
   p->numflds = sql_num_fields(res); /* get number of fields */
   p->fields = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * p->numflds);
   
   for(x=0; x<p->numflds;x++)
   {
      SQL_FIELD *field;
      
      field = sql_fetch_field(res);
      
      /* copy the complete field struct to an array element : */
      memcpy(&p->fields[x],field,sizeof(SQL_FIELD));

      // FIX ME :
      /* This is just a kludge : integer types only have a length of 2 set,
       * which is not enough to hold the data as a string value, which is
       * what Gequel does. For now I just set it to a larger value of 16. */
    
      /* Dec 2011 : consider using mysql_fetch_lengths() ? */
      
      ldebug(4,"field: %s, old length: %d\n",p->fields[x].name,p->fields[x].length);
      if(IS_NUM(p->fields[x].type))
	p->fields[x].length = 16;
      ldebug(4,"field: %s, new length: %d\n",p->fields[x].name,p->fields[x].length);
      
      /* Now save (copy) some pointer elements (name and def as they'll 
       * be lost after release of the selection , because memcpy
       * doesn't copy the actual content but only the reference ! */
      
      /* Note that we do not copy all strings here, so fieldinfo 
       * is basically not complete (e.g. table is missing) ! */
      
      tlen = strlen(p->fields[x].name) + 1;
      p->fields[x].name = (char *)g_malloc(tlen);
      strncpy(p->fields[x].name, field->name, tlen);

      if(p->fields[x].def != NULL)
      {
	 tlen = strlen(p->fields[x].def) + 1;
	 p->fields[x].def = (char *)g_malloc(tlen);
	 strncpy(p->fields[x].def, field->def, tlen);
      }
   }
   
   p->sortkey = getSortKey();
   setGUICounters(p->numflds, p->numrecs);
   sql_free_result(res);
}

/* Remove the table list, clear the previous table and set counters to 0 */
void clearTableList(void)
{
   Params *p = getParams();
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   
   ldebug(2,"clearTableList called\n");
   p->table[0] = 0;
   
   releaseFieldInfo(); /* Free previous field info to avoid memory leaks */
   
   gtk_option_menu_remove_menu (p->tablelist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* None *");
   gtk_widget_show (menulabel);

   gtk_menu_append(GTK_MENU(newmenu), menulabel);   
   gtk_option_menu_set_menu(p->tablelist,newmenu);
}

/* Remove database list and previous database and then call clearTableList
 to remove the table list and to clear the previous table */
void clearDBaseList(void)
{
   Params *p = getParams();
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   
   ldebug(2,"clearDBaseList called\n");
   clearTableList();
   p->database[0] = 0;

   gtk_option_menu_remove_menu (p->dbaselist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* None *");
   gtk_widget_show (menulabel);

   gtk_menu_append(GTK_MENU(newmenu), menulabel);   
   gtk_option_menu_set_menu(p->dbaselist,newmenu);
   
}

/* Clear previous table and refresh the table list to contain the one obtained
 from the currently selected database, optionally preselected a table */
void setTableList(char *preselect)
{
   int       counter, preset = 0;
   SQL_RES   *res;
   SQL_ROW    row;
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   Params *p = getParams();
   
   ldebug(2,"setTableList called\n");
   p->table[0] = 0;
   releaseFieldInfo(); /* Free previous field info to avoid memory leaks */
   
   if (p->dbsock == NULL)
   {
      showMessage("Cannot fetch tables: no connection");
      return;
   }
   
   res = sql_list_tables(p->dbsock,0);
   if(res == NULL)
   {
      showSqlError(p->dbsock,"Cannot fetch tables for database %s",
		   p->database);
      return;
   }
   
   gtk_option_menu_remove_menu(p->tablelist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* Select *");
   gtk_widget_show (menulabel);
   gtk_menu_append(GTK_MENU(newmenu), menulabel);

   counter = 1;
   while((row = sql_fetch_row(res)) != NULL)
   {
      menulabel = gtk_menu_item_new_with_label(row[0]);
      gtk_widget_show (menulabel);
      gtk_menu_append(GTK_MENU(newmenu), menulabel);
      gtk_signal_connect
	(GTK_OBJECT (menulabel), "activate",
	 GTK_SIGNAL_FUNC (on_table_activate),
	 (gpointer) p);
      
      /* set current table to preselect in optionmenu */
      if(preselect != NULL)
      {
	 if(strcmp(row[0],preselect) == 0)
	   preset = counter;
	 counter++;
      }
      
      ldebug(3,"table: %s\n",row[0]);
   }
   gtk_option_menu_set_menu(p->tablelist,newmenu);
   gtk_option_menu_set_history (p->tablelist, preset);
   
   sql_free_result(res);
}

/* Call clearTableList to clear the table list and previous table, clear the
 curent database and refresh the database list to contain the one obtained
 from the current server */
void setDBaseList(char *preselect)
{
   int       counter, preset = 0;
   SQL_RES    *res;
   SQL_ROW    row;
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   Params *p = getParams();
     
   ldebug(2,"setDBaseList called\n");
   p->database[0] = 0;
   clearTableList();
   
   if (p->dbsock == NULL)
   {
      showMessage("Cannot fetch databases: no connection");
      return;
   }
   
   res = sql_list_dbs(p->dbsock,0);
   if(res == NULL)
   {
      showSqlError(p->dbsock,"Cannot fetch databases from server"
		   " %s",p->server == NULL ? p->socketname : p->server);
      return;
   }
   
   gtk_option_menu_remove_menu (p->dbaselist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* Select *");
   gtk_widget_show (menulabel);
   gtk_menu_append(GTK_MENU(newmenu), menulabel);
   
   counter = 1;
   while((row = sql_fetch_row(res)) != NULL)
   {
      menulabel = gtk_menu_item_new_with_label(row[0]);
      gtk_widget_show (menulabel);
      gtk_menu_append(GTK_MENU(newmenu), menulabel);
      gtk_signal_connect
	(GTK_OBJECT (menulabel), "activate",
	 GTK_SIGNAL_FUNC (on_database_activate),
	 (gpointer) p);
      
      if(preselect != NULL)
      {
	 if(strcmp(row[0],preselect) == 0)
	    preset = counter;
	 counter++;
      }
      
      ldebug(3,"database: %s\n",row[0]);
   }
   gtk_option_menu_set_menu(p->dbaselist,newmenu);
   gtk_option_menu_set_history (p->dbaselist, preset);

   sql_free_result(res);
}

/* Select a new table, read table info (fields) and set the GUI counters */
void setNewTable(char *table)
{
   Params *p = getParams();
   
   ldebug(2,"setNewTable called\n");
   clearAllTabs();

   strncpy(p->table,table,MAXSTRING-1);
   
   if(table == NULL)
      return;
   
   getNumRecords();
   getFieldInfo();
   
   forceRefreshRecordTab(); // call notebook callback
}

/* Use a new database, clear the previous table, fill the tablelist
 * with tables from this database and set the table counters to 0 */
void setNewDatabase(char *database)
{
   Params *p = getParams();
   
   ldebug(2,"setNewDatabase called\n");
   clearAllTabs();

   strncpy(p->database,database,MAXSTRING -1 );

   if(database == NULL)
     return;
   
   if(sql_select_db(p->dbsock,database) != 0)
   {
      showSqlError(p->dbsock,"Error opening database %s",database);
      clearTableList();
      return;
   }
   p->numrecs = 0;
   
   setTableList(NULL);
}

/* Connect to a database server, set the server and list of available
 databases, clear tablelist and clear previous table and database.
 A NULL server will connect to the localhost using a unix socket. */
void setNewServer(char *server, char *socketname)
{
   Params *p = getParams();
   
   ldebug(2,"setNewServer called\n");
   clearAllTabs();
   
   p->server = server;
   if(p->server != NULL)
   {
      if (strlen(p->server) == 0)
	p->server = NULL;  /* provide for empty but not NULL string */
      else
	setStringValue(p->mainwindow, "server", p->server);
   }

   p->server = server;
   if(p->server != NULL)
   {
      if (strlen(p->server) == 0)
	p->server = NULL;  /* provide fix for empty but not NULL string , causing a segfault */
      else
	setStringValue(p->mainwindow, "server", p->server);
   }

   p->socketname = socketname;
   if(p->socketname != NULL)
   {
      if (strlen(p->socketname) == 0)
	p->socketname = DEFSOCKETNAME;  
   }

   p->dbsock=mysql_init(NULL);
   if (p->dbsock == NULL || mysql_real_connect
       (p->dbsock, p->server, p->user, p->pwd, NULL, 0, p->socketname, 0) == NULL)
   {
      showSqlError(p->dbsock, "Error connecting to server");
      clearDBaseList();
   }
   else
     setDBaseList(NULL);
}

